package com.totopia.server.modules.user.service;

import com.totopia.server.auth.payload.SignupRequest;
import com.totopia.server.commons.exception.ResourceNotFoundException;
import com.totopia.server.modules.user.entity.RoleEntity;
import com.totopia.server.modules.user.entity.UserEntity;
import com.totopia.server.modules.user.payload.UserUpdateRequest;
import com.totopia.server.modules.user.repository.RoleRepository;
import com.totopia.server.modules.user.repository.UserRepository;
import com.totopia.server.modules.user.type.RoleName;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Service;

import java.util.Collections;
import java.util.Date;

@Service
public class UserService {

    @Autowired
    private UserRepository userRepository;

    @Autowired
    RoleRepository roleRepository;

    @Autowired
    PasswordEncoder passwordEncoder;

    public Page<UserEntity> getUsersByPageable(String username, Pageable pageable) {
        Page<UserEntity> entities = null;

        if (null == username) {
            entities = userRepository.findAll(pageable);
        } else {
            entities = userRepository.findByUsernameContainingIgnoreCase(username, pageable);
        }

        return entities;
    }

    public Boolean existUserByUsername(String username) {
        return this.userRepository.existsByUsername(username);
    }

    public UserEntity getUserById(Long id) {
        UserEntity userEntity = this.userRepository.findById(id)
                .orElseThrow( () -> new ResourceNotFoundException("User", "userId", id));

        return userEntity;
    }

    public Long removeUserById(Long userId) {
        return userRepository.findById(userId).map(user -> {
            userRepository.delete(user);
            return userId;
        }).orElseThrow(() -> new ResourceNotFoundException("User", "userId", userId));
    }

    public void regist(SignupRequest signupRequest) throws Exception{


        // Creating user's account
        UserEntity user = UserEntity.builder().username(signupRequest.getUsername()).email(signupRequest.getEmail())
                .nickname(signupRequest.getNickname()).phone(signupRequest.getPhone()).descriptions(signupRequest.getDescriptions())
                .password(signupRequest.getPassword()).block(false).resetCount(0L).sendEmail(true).build();

        user.setPassword(passwordEncoder.encode(user.getPassword()));

        RoleEntity userRole = roleRepository.findByName(RoleName.ROLE_USER)
                .orElseThrow(() -> new Exception("User Role not set."));

        user.setRoles(Collections.singleton(userRole));

        UserEntity result = userRepository.save(user);
    }

    public UserEntity modifyByUser(UserEntity existEntity, UserUpdateRequest updateRequest) {

        String existNickname = existEntity      .getNickname();
        String existPhone =  existEntity        .getPhone();
        String existEmail = existEntity         .getEmail();
        String existPassword = existEntity      .getPassword();
        String existDescriptions = existEntity  .getDescriptions();

        String newNickname = updateRequest.getNickname();
        String newPhone =  updateRequest.getPhone();
        String newEmail = updateRequest.getEmail();
        String newPassword = updateRequest.getPassword();
        String newDescriptions = updateRequest.getDescriptions();

        if (newNickname.equals(existNickname)
                && newPhone.equals(existPhone)
                && newEmail.equals(existEmail)
                && this.passwordEncoder.matches(newPassword,existPassword)
                && newDescriptions.equals(existDescriptions)
        ) {
            return existEntity;
        }

        if (!newNickname.equals("") && !newNickname.equals(existNickname)) {
            existEntity.setNickname(newNickname);
        }
        if (!newPhone.equals("") && !newPhone.equals(existPhone)) {
            existEntity.setPhone(newPhone);
        }
        if (!newEmail.equals("") && !newEmail.equals(existEmail)) {
            existEntity.setEmail(newEmail);
        }

        if (!newPassword.equals("") && !this.passwordEncoder.matches(newPassword,existPassword)) {
            existEntity.setPassword(passwordEncoder.encode(newPassword));
            existEntity.setLastResetTime(new Date());
        }
        if (!newDescriptions.equals("") && !newDescriptions.equals(existDescriptions)) {
            existEntity.setDescriptions(newDescriptions);
        }

        existEntity.setUpdatedAt(new Date());

        existEntity = this.userRepository.save(existEntity);

        return existEntity;
    }
}
